<?php

declare(strict_types=1);

namespace Dedi\SyliusMarketingPlugin\Storage;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

class SessionManager implements SessionManagerInterface
{
    private RequestStack $requestStack;

    public function __construct(RequestStack $requestStack)
    {
        $this->requestStack = $requestStack;
    }

    public function storeInSession(string $key, array $data): void
    {
        $currentRequest = $this->getCurrentRequest();
        if (!$currentRequest instanceof Request) {
            return;
        }

        $session = $currentRequest->getSession();
        $storedData = $session->get($key) ?? [];
        $session->set($key, array_merge($storedData, $data));
    }

    public function getFromSession(string $key)
    {
        $currentRequest = $this->getCurrentRequest();
        if (!$currentRequest instanceof Request) {
            return null;
        }

        return $currentRequest->getSession()->get($key);
    }

    private function getCurrentRequest(): ?Request
    {
        return $this->requestStack->getCurrentRequest();
    }
}
